--SCHEMA
CREATE SCHEMA "BellItalia";


--HOTEL

CREATE TABLE "BellItalia".Hotel
(
  mail character varying(32) NOT NULL,
  fax numeric(10,0),
  orario character varying(23) NOT NULL,
  categoria numeric(1) NOT NULL,
  via character varying(32) NOT NULL,
  citta character varying(32) NOT NULL,
  numero numeric(3,0) NOT NULL,
  gestore integer,
  --non NOT NULL perche non esiste tabella personale ancora
  CONSTRAINT Hotel_pkey PRIMARY KEY (mail),
  CONSTRAINT categoria_check CHECK (categoria = 3::numeric OR categoria = 4::numeric)
);

--TEL

CREATE TABLE "BellItalia".Tel
(
  numero numeric(10,0) NOT NULL,
  hotel character varying(32) NOT NULL,
  CONSTRAINT Tel_pkey PRIMARY KEY (numero, hotel),
  CONSTRAINT Tel_Hotel_fkey FOREIGN KEY (hotel)
     REFERENCES "BellItalia".Hotel (mail) MATCH SIMPLE
     ON UPDATE CASCADE ON DELETE CASCADE
);

--PIANI

CREATE TABLE "BellItalia".Piani
(
  numero numeric(2,0) NOT NULL,
  CONSTRAINT numero_check CHECK (numero > 0::numeric),
  hotel character varying(32) NOT NULL,
  CONSTRAINT Piani_pkey PRIMARY KEY (numero, hotel),
  CONSTRAINT Piani_Hotel_fkey FOREIGN KEY (hotel)
     REFERENCES "BellItalia".Hotel (mail) MATCH SIMPLE
     ON UPDATE CASCADE ON DELETE CASCADE,
  numStanze numeric(2,0) NOT NULL,
  CONSTRAINT numStanze_check CHECK (numStanze > 0::numeric),
  numAscensori numeric(1,0) NOT NULL DEFAULT 1,
  CONSTRAINT numAscensori_check CHECK (numAscensori >= 1::numeric),
  numScale numeric(1,0) NOT NULL,
  CONSTRAINT numScale_check CHECK (numScale >= 1::numeric),
  numVieFuga numeric(1,0) NOT NULL,
  CONSTRAINT numVieFuga_check CHECK (numVieFuga >= 1::numeric),
  referente integer
  --non NOT NULL perche non esiste tabella personale ancora
);

--CAMERE

CREATE TABLE "BellItalia".Camere
(
  numero numeric(2,0) NOT NULL,
  CONSTRAINT numero_check CHECK (numero > 0::numeric),
  hotel character varying(32) NOT NULL,
  piano numeric(2,0) NOT NULL,
  CONSTRAINT Camere_pkey PRIMARY KEY (numero, hotel, piano),
  CONSTRAINT Camere_Hotel_fkey FOREIGN KEY (hotel, piano)
     REFERENCES "BellItalia".Piani (hotel, numero) MATCH SIMPLE
     ON UPDATE CASCADE ON DELETE CASCADE,
  lato character NOT NULL,
  CONSTRAINT lato_check CHECK (lato = 'w'::character OR lato = 's'::character OR lato = 'e'::character OR lato = 'n'::character),
  balconi numeric(1,0),
  CONSTRAINT balconi_check CHECK (balconi >=0::numeric),
  superficie numeric(5,2) NOT NULL,
  CONSTRAINT superficie_check CHECK (superficie >0::numeric),
  disabili boolean NOT NULL DEFAULT FALSE,
  prezzoMin numeric(5,2) NOT NULL,
  CONSTRAINT prezzoMin_check CHECK(prezzoMin >0::numeric),
  prezzoMax numeric(5,2) NOT NULL,
  CONSTRAINT prezzoMax_check CHECK(prezzoMax >0::numeric),
  CONSTRAINT prezzo_check CHECK(prezzoMin <= prezzoMax),
  tipobagno character varying(13) NOT NULL,
  CONSTRAINT tipobagno_check CHECK (tipobagno = 'doccia'::character varying OR tipobagno = 'vasca'::character varying OR tipobagno = 'idromassaggio'::character varying), 
  suite boolean DEFAULT FALSE,
  tipocamera character varying(7) NOT NULL,
   CONSTRAINT tipocamera_check CHECK (tipocamera = 'singola'::character varying OR tipocamera = 'doppia'::character varying) 
);

--PERSONALE

CREATE TABLE "BellItalia".Personale
(
  CodP integer NOT NULL,
  CONSTRAINT Personale_pkey PRIMARY KEY (CodP),
  nome character varying(32) NOT NULL,
  cognome character varying(32) NOT NULL,
  telefono numeric(10,0) NOT NULL,
  dataNascita date NOT NULL,
  indirizzo character varying(32),
  dataAssunzione date NOT NULL,
  dataArrivo date NOT NULL,
  dataInizio date NOT NULL,
  dataFine date,
  CONSTRAINT data_check CHECK (dataAssunzione <= dataArrivo),
  CONSTRAINT data_check1 CHECK (dataArrivo <= dataInizio),
  CONSTRAINT data_check2 CHECK (dataInizio < dataFine),
  mansione character varying(32) NOT NULL,
  orario character varying(23) NOT NULL,
  stipendio numeric(5,0) NOT NULL,
  CONSTRAINT stipendio_check CHECK (stipendio > 0::numeric),
  hotel character varying(32) NOT NULL,
  CONSTRAINT Personale_Hotel_fkey FOREIGN KEY (hotel)
     REFERENCES "BellItalia".Hotel (mail) MATCH SIMPLE
     ON UPDATE CASCADE ON DELETE CASCADE
);
--aggiunge le hiavi esterne che prima non poteva aggiungere, da fare dopo aver creato hotel e piano!!!
ALTER TABLE "BellItalia".Piani ADD CONSTRAINT Piani_PersonaleReferente_fkey FOREIGN KEY (referente)
  REFERENCES "BellItalia".Personale (CodP) MATCH SIMPLE
  ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE "BellItalia".Hotel ADD CONSTRAINT Hotel_PersonaleGestore_fkey FOREIGN KEY (gestore)
  REFERENCES "BellItalia".Personale (CodP) MATCH SIMPLE
  ON UPDATE CASCADE ON DELETE CASCADE;
  
  --CLIENTE

CREATE TABLE "BellItalia".Cliente
(
  CodC integer NOT NULL,
  nome character varying(32) NOT NULL,
  cognome character varying(32) NOT NULL,
  telefono numeric(10,0) NOT NULL,
  dataNascita date NOT NULL,
  indirizzo character varying(32) NOT NULL,
  CONSTRAINT Cliente_pkey PRIMARY KEY (CodC)
);

--PRENOTA

CREATE TABLE "BellItalia".Prenota
(
  numero numeric(2,0) NOT NULL,
  piano numeric(2,0) NOT NULL,
  hotel character varying(32) NOT NULL,
  CodC integer NOT NULL,
  CONSTRAINT Prenota_pkey PRIMARY KEY (numero, piano, hotel, CodC),
  CONSTRAINT Prenota_Camera_fkey FOREIGN KEY (numero, piano, hotel)
     REFERENCES "BellItalia".Camere (numero, piano, hotel) MATCH SIMPLE
     ON UPDATE CASCADE ON DELETE CASCADE,
  CONSTRAINT Prenota_Cliente_fkey FOREIGN KEY (CodC)
     REFERENCES "BellItalia".Cliente (CodC) MATCH SIMPLE
     ON UPDATE CASCADE ON DELETE CASCADE,
  pagamento character varying(10) NOT NULL,
  CONSTRAINT pagamento_check CHECK (pagamento = 'contanti'::character varying OR pagamento = 'bonifico'::character varying OR pagamento = 'carta'::character varying OR pagamento = 'assegno'::character varying),
  dataArrivo date NOT NULL,
  dataPartenza date NOT NULL,
  CONSTRAINT data_check CHECK (dataArrivo <= dataPartenza)
);

--RISTORANTE

CREATE TABLE "BellItalia".Ristorante
(
  hotel character varying(32) NOT NULL,
  CONSTRAINT Ristorante_pkey PRIMARY KEY (hotel),
  CONSTRAINT hotel_Hotel_fkey FOREIGN KEY (hotel)
	REFERENCES "BellItalia".Hotel (mail) MATCH SIMPLE
	 ON UPDATE CASCADE ON DELETE CASCADE,
  posti numeric(3,0) NOT NULL,
  CONSTRAINT posti_check CHECK (posti >0::numeric),
  orarioColazione character varying(11) NOT NULL,
  orarioPranzo character varying(11) NOT NULL,
  orarioCena character varying(11) NOT NULL
  CONSTRAINT orario_check CHECK (orarioColazione < orarioPranzo),
  CONSTRAINT orario_check1 CHECK (orarioPranzo < orarioCena)
);

--LavoraIn

CREATE TABLE "BellItalia".LavoraIn
(
  CodP integer NOT NULL,
  ristorante character varying(32) NOT NULL,
  CONSTRAINT LavoraIn_pkey PRIMARY KEY (CodP, ristorante),
  CONSTRAINT CodP_PersonaleLavoraIn_fkey FOREIGN KEY (CodP)
	REFERENCES "BellItalia".Personale (CodP) MATCH SIMPLE
	 ON UPDATE CASCADE ON DELETE CASCADE,
  CONSTRAINT ristorante_RistoranteLavoraIn_fkey FOREIGN KEY (ristorante)
	REFERENCES "BellItalia".Ristorante (hotel) MATCH SIMPLE
	 ON UPDATE CASCADE ON DELETE CASCADE
);

--LINGUE

CREATE TABLE "BellItalia".Lingue
(
  lingua character varying(32) NOT NULL,
  CONSTRAINT Lingue_pkey PRIMARY KEY (lingua)
);

--PARLA

CREATE TABLE "BellItalia".Parla
(
  CodP integer NOT NULL,
  lingua character varying(32) NOT NULL,
  CONSTRAINT Parla_pkey PRIMARY KEY (lingua, CodP),
  CONSTRAINT CodP_Personale_fkey FOREIGN KEY (CodP)
      REFERENCES "BellItalia".Personale (CodP) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE,
  CONSTRAINT lingua_Lingue_fkey FOREIGN KEY (lingua)
      REFERENCES "BellItalia".Lingue (lingua) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE
);

--PARCHEGGIO

CREATE TABLE "BellItalia".Parcheggio
(
  ID integer NOT NULL,
  hotel character varying(32) NOT NULL,
  CONSTRAINT Parcheggio_pkey PRIMARY KEY (ID),
  CONSTRAINT hotel_Parcheggio_fkey FOREIGN KEY (hotel)
	REFERENCES "BellItalia".Hotel (mail) MATCH SIMPLE
	 ON UPDATE CASCADE ON DELETE CASCADE,
  quota numeric(5,2),
  CONSTRAINT quota_check_min CHECK (quota>=2),
  CONSTRAINT quota_check_max CHECK (quota<=20),
  coperto boolean NOT NULL
);

--PISCINA

CREATE TABLE "BellItalia".Piscina
(
  ID integer NOT NULL,
  hotel character varying(32) NOT NULL,
  CONSTRAINT Piscina_pkey PRIMARY KEY (ID),
  CONSTRAINT hotel_Piscina_fkey FOREIGN KEY (hotel)
	REFERENCES "BellItalia".Hotel (mail) MATCH SIMPLE
	 ON UPDATE CASCADE ON DELETE CASCADE,
  coperto boolean NOT NULL,
  profondita numeric(3,2) NOT NULL
);

--FITNESS

CREATE TABLE "BellItalia".Fitness
(
  ID integer NOT NULL,
  hotel character varying(32) NOT NULL,
  CONSTRAINT Fitness_pkey PRIMARY KEY (ID),
  CONSTRAINT hotel_Fitness_fkey FOREIGN KEY (hotel)
	REFERENCES "BellItalia".Hotel (mail) MATCH SIMPLE
	 ON UPDATE CASCADE ON DELETE CASCADE,
  tipo character varying(8) NOT NULL,
  CONSTRAINT tipo_check CHECK (tipo = 'tennis'::character varying OR tipo = 'golf'::character varying OR tipo = 'palestra'::character varying OR tipo = 'sauna'::character varying),
  apertura character varying(11) NOT NULL,
  materiale character varying(32),
  quota numeric(5,2),
  buche numeric(2)
);
